# Asca
[![Circle CI](https://circleci.com/gh/Originate/asca/tree/master.svg?style=svg)](https://circleci.com/gh/Originate/asca/tree/master)

Convenient parameter binding for asynchronous functions.

1. load asca

  ```coffeescript
  asca = require 'asca'
  ```


2. define asynchronous functions using asca


  ```coffeescript

  sayHi = asca (name, done) ->
    console.log "Hello #{name}"
    done()
  ```

3. the `sayHi` method behaves completely normal if called normally,
   i.e. given an asynchronous callback

  ```coffeescript
  sayHi 'world', ->   # "Hello world"
  ```

4. if called without the asynchronous callback,
  the `sayHi` method returns a method with all the parameters bound to it.

  ```coffeescript
  sayLater = sayHi 'world'   # 'sayLater' is a method with the argument
                             # 'world' bound to it
  sayLater ->                # "Hello world"
  ```

5. Use the bound form for very readable asynchronous code constructs,
  for example when using [async](https://github.com/caolan/async):

  ```coffeescript
  # instead of
  async.parallel [
    (done) -> sayHi 'world', done
    (done) -> sayHi 'universe', done
  ]

  # we can now say
  async.parallel [
    sayHi 'world'
    sayHi 'universe'
  ]
  ```

  Or other places:
  ```coffeescript
  # instead of
  setTimeout ((done) -> sayHi 'world', done), 2000

  # we can now say
  setTimeout sayHi('world'), 2000
  ```

  There are other libraries like [curry](https://www.npmjs.com/package/curry)
  that provide more comprehensive currying and binding,
  and might be more appropriate for your intended usage.

  This library is focussed around asynchrounous function calling,
  for example through error checking specifically for that use case.
  It is also extremely lightweight and performant.
